unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, Grids, RXCtrls, Menus, ImgEdt, ExtCtrls, ToolWin,
  ComCtrls;

type
  TForm1 = class(TForm)
    OpenDialog1: TOpenDialog;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    Save1: TMenuItem;
    Saveas1: TMenuItem;
    Close1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    SaveDialog1: TSaveDialog;
    ScrollBox1: TScrollBox;
    ImgEditor1: TImgEditor;
    ToolBar1: TToolBar;
    ToolBar2: TToolBar;
    ImageList1: TImageList;
    ImageList2: TImageList;
    btnPencil: TToolButton;
    btnBrush: TToolButton;
    Panel1: TPanel;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    btnZoom: TToolButton;
    btnEyeDrop: TToolButton;
    CoolBar1: TCoolBar;
    ToolBar3: TToolBar;
    CheckBox1: TCheckBox;
    ColorDialog1: TColorDialog;
    Panel2: TPanel;
    pnlForeColor: TPanel;
    pnlBackColor: TPanel;
    StatusBar1: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure Saveas1Click(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);

    Procedure SetPixel (X, Y : Integer; Color : TColor);
    procedure ImgEditor1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ImgEditor1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ImgEditor1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pnlForeColorDblClick(Sender: TObject);
    procedure pnlBackColorDblClick(Sender: TObject);
  private
    { Private declarations }
    FileName : String;
    Changed : Boolean;
    Dragging : Boolean;
    Procedure Reset;
    Function SaveImage : Boolean;   {Returns true if image is really saved}
    Function SaveImageAs : Boolean; {Returns true if image is really saved}
    Function MaybeSaveImage : Boolean; {Returns true if image is really saved}
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

Const DefCap : String [7] = 'ImgEdit';

procedure TForm1.FormCreate(Sender: TObject);
begin
  Dragging := False;
  Reset;
end;

procedure TForm1.Reset;
Begin
  FileName := '';
  ImgEditor1.Bitmap.Width := 0;
  ImgEditor1.Bitmap.Height := 0;
  Caption := DefCap;
  Changed := False;
End;

Function TForm1.SaveImage : Boolean;   {Returns true if image is really saved}
Begin
  If FileName = '' Then
    Result := SaveImageAs
  Else Begin
    ImgEditor1.Bitmap.SaveToFile (FileName);
    Changed := False;
    Result := True;
  End;
End;

Function TForm1.SaveImageAs : Boolean; {Returns true if image is really saved}
Begin
  If SaveDialog1.Execute Then Begin
    FileName := SaveDialog1.FileName;
    ImgEditor1.Bitmap.SaveToFile (FileName);
    Caption := DefCap + ' - ' + FileName;
    Changed := False;
    Result := True;
  End
  Else
    Result := False;
End;

Function TForm1.MaybeSaveImage : Boolean; {Returns true if image is really saved}
Begin
  Case MessageDlg ('Save changes ?', mtConfirmation, mbYesNoCancel, 0) of
    mrYes : Result := SaveImage;
    mrNo  : Result := True;
    Else    Result := False;
  End;
End;

Procedure TForm1.SetPixel (X, Y : Integer; Color : TColor);
Begin
  If ImgEditor1.Bitmap.Empty Then Exit;

  If ImgEditor1.Bitmap.Canvas.Pixels [X, Y] <> Color Then Begin
    ImgEditor1.Bitmap.Canvas.Pixels [X, Y] := Color;
    {If using invalidate, it flickers like hell!
     Guess it first clears the area before redrawing it.
     Now it still flickers a little :(
     I tried to Use a temporary bitmap in the paint method, bit this
     is sloooow! so, this seems the best option... any ideas welcome!}
    ImgEditor1.Paint;
    Changed := True;
  End;
End;

procedure TForm1.CheckBox1Click(Sender: TObject);
begin
  ImgEditor1.ShowGrid := CheckBox1.Checked;
end;

procedure TForm1.Open1Click(Sender: TObject);
begin
  If (not Changed) or (Changed and MaybeSaveImage) Then
    If OpenDialog1.Execute Then Begin
      FileName := OpenDialog1.FileName;
      ImgEditor1.Bitmap.LoadFromFile (FileName);
      {Don't know a way of notifying ImgEditor of the fact that the bitmap
       has changed, so we have to call this method ourselves... any hints?}
      ImgEditor1.RecalcSize;

      Changed := False;

      Caption := DefCap + ' - ' + FileName;
    End;
end;

procedure TForm1.Save1Click(Sender: TObject);
begin
  SaveImage;
end;

procedure TForm1.Saveas1Click(Sender: TObject);
begin
  SaveImageAs;
end;

procedure TForm1.Close1Click(Sender: TObject);
begin
  If (not Changed) or (Changed and MaybeSaveImage) Then Reset;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := (not Changed) or (Changed and MaybeSaveImage);
end;

procedure TForm1.ImgEditor1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
Var NewCol : TColor;
begin
  If btnPencil.Down Then Begin {Pencil tool selected}
    Case Button of
      mbLeft  : NewCol := pnlForeColor.Color;
      mbRight : NewCol := pnlBackColor.Color;
      Else      Exit;
    End;
    SetPixel (X, Y, NewCol);
    Dragging := True;
  End Else If btnPencil.Down Then Begin {Brush tool selected}
    {nothing yet..}
  End Else If btnZoom.Down Then Begin {Zoom tool selected}
    Case Button of
      mbLeft  : ImgEditor1.PixelSize := ImgEditor1.PixelSize * 2;
      mbRight : ImgEditor1.PixelSize := ImgEditor1.PixelSize div 2;
      Else      Exit;
    End;
  End Else If btnEyeDrop.Down Then Begin {Eyedropper tool selected}
    Case Button of
      mbLeft  : pnlForeColor.Color := ImgEditor1.Bitmap.Canvas.Pixels [X, Y];
      mbRight : pnlBackColor.Color := ImgEditor1.Bitmap.Canvas.Pixels [X, Y];
      Else      Exit;
    End;
  End;
end;

procedure TForm1.ImgEditor1MouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
Var NewCol   : TColor;
begin
  If btnPencil.Down Then Begin {Pencil tool selected}
    If not Dragging Then Exit;

    If ssLeft in Shift Then NewCol := pnlForeColor.Color
    Else If ssRight in Shift Then NewCol := pnlBackColor.Color
    Else Exit;
    SetPixel (X, Y, NewCol);
  End Else If btnPencil.Down Then Begin {Brush tool selected}
    {nothing yet..}
  End Else If btnZoom.Down Then Begin {Zoom tool selected}
    {Doesn't make sense}
  End Else If btnEyeDrop.Down Then Begin {Eyedropper tool selected}
    {Doesn't make sense}
  End;
end;

procedure TForm1.ImgEditor1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
Var NewCol   : TColor;
begin
  If btnPencil.Down Then Begin {Pencil tool selected}
    Case Button of
      mbLeft  : NewCol := pnlForeColor.Color;
      mbRight : NewCol := pnlBackColor.Color;
      Else      Exit;
    End;
    SetPixel (X, Y, NewCol);
    Dragging := False;
  End Else If btnPencil.Down Then Begin {Brush tool selected}
    {nothing yet..}
  End Else If btnZoom.Down Then Begin {Zoom tool selected}
    {Doesn't make sense}
  End Else If btnEyeDrop.Down Then Begin {Eyedropper tool selected}
    {Doesn't make sense}
  End;
end;

procedure TForm1.pnlForeColorDblClick(Sender: TObject);
begin
  ColorDialog1.Color := pnlForeColor.Color;
  if ColorDialog1.Execute then
      pnlForeColor.Color := ColorDialog1.Color;
end;

procedure TForm1.pnlBackColorDblClick(Sender: TObject);
begin
  ColorDialog1.Color := pnlBackColor.Color;
  if ColorDialog1.Execute then
      pnlBackColor.Color := ColorDialog1.Color;
end;

end.
